function [ nll,grad ] = RMCMIL_Loss( p , X, yi,instancesBagIdxG,k,r )
%RMCMIL_LossHyperplane Computes Loss and Gradient of RMC-MIL optimization
%modelling concepts as hyperplanes in the instace space.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Gather model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
p=p';
[N D] = size(X);
W = p(1:k+1);
z = p(k+2:end);
Z = reshape(z,D,k);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Instance concept probability
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T = -Z'*X';
pijk = double(1./(1 + exp(T')));
clear T;
pijk = max(min(pijk,1-eps),eps); %avoid numerical problems


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Bag-Concept Embedding
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Generalized Mean for each concepts
pijkr = ot_integerPow2(pijk,r);
GG = inv(instancesBagIdxG*instancesBagIdxG');
R = GG*instancesBagIdxG;
Gir = R*pijkr;
Gi = power(Gir,1/pow2(r-1));
%Bag representations
Gi = [Gi ones(size(Gi,1),1)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Bag Probability
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
py = W*Gi';
py = 1./(1+exp(-py));
py = max(min(py,1-eps),eps); %avoid numerical problems
err = (yi.*log(py)+(1-yi).*log(1-py));
nll= -double(sum(err));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Compute Gradients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargout > 1
    bagWeights = (yi - py);
    %%% Bag-Classifier Gradient
    wPij = bsxfun(@times,Gi,bagWeights');
    wgrad = -(sum(wPij));
    
    %%% Hyperplane parameters gradients
    bagWeightsK = repmat(full(diag(GG)).*bagWeights',1,k).*(Gi(:,1:end-1)./(Gir+eps));
    bagWeightsK = instancesBagIdxG'*bagWeightsK;
    wij =  pijkr.*(1-pijk);
    clear pijkr pijk;
    gradZ = -X'*(wij.*bagWeightsK);
    clear wij;
    gradZ = gradZ.*repmatC(W(1:end-1)',1,size(gradZ,1))';
    
    %Return model parameters gradient
    grad = double([wgrad(:)' gradZ(:)']');
end




end

