function w = RMCMIL_Projection(p,beta,groups)
%RMCMIL_ProjectionGaussian Projects the hyperplane parameters onto the
%L21-norm ball according to parameter b. Used in the optimization of RMCMIL
%with the Projected-Quasi-Newton method

p=p';
ind = groups~=0;
d = max(groups);
k = sum(ind) / d;
w = p(1:k)';
z = p(k+2:end);
Z = reshape(z,d+1,k);
norms = sqrt(accumarray(groups(ind),z(ind).^2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% L21 projection 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
if sum(norms) > beta
    projectedNorms = projectRandom2C(norms,beta*d);
    norms(norms==0) = 1;
    z = z.*repmat([(projectedNorms./norms)' 1],1,k);
    Z = reshape(z,d+1,k);
end
w = [w' p(k+1) Z(:)']';
