function [ bagLabels bagProbs ] = RMCMIL_Test( model, bags, instancesBagIdx )
%RMCMIL_TestHyperplane Classify bags using a leraned RMC-MIL model using concepts modelled as hyperplanes in the instance space.
%   Call:
%       model = RMCMIL_TestHyperplane( model, bags, instancesBagIdx, params ) returns the parameters of the RMCMIL model
%   Input:
%       model: Trained RMC-MIL model
%
%       bags: NxD Matrix with N samples of D dimensions contained in all the bags
%
%       instancesBagIdx: Nx1 vector indicating what bag the instance n
%       belongs to. Contains integers from 1 to M, where M is the total
%       number of bags
%       
%   Output:
%       bagLabels: Estimated bag labels
%       bagLabels: Estimated bag probabilities
%     


%%% Feafures standarization
model.std = model.std(1:size(bags,2));
model.mean = model.mean(:,1:size(bags,2));
model.std(model.std==0) = 1;
bags = bsxfun(@minus,bags, model.mean);
bags = bsxfun(@rdivide, bags, model.std);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% Gather Model Parameters 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
W = model.W;
Z = model.Z(1:(size(bags,2)+1),:);
r = model.r;
instancesBagIdxG = ot_idx2G(instancesBagIdx,max(instancesBagIdx));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Instance concept probability
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pijk= double(1./(1 + exp(-Z'*[bags ones(size(bags,1),1)]')));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Bag-Concept Embedding  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Generalized Mean for each concepts
R=inv(instancesBagIdxG*instancesBagIdxG')*instancesBagIdxG;
Gi = R*ot_integerPow2(pijk',r);
Gi = power(Gi,1/pow2(r-1));
Gi = [Gi ones(size(Gi,1),1)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Bag probability and labels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
py = W'*Gi';
bagProbs = 1./(1+exp(-py));
bagLabels = bagProbs>0.5;

end

