clear all


%%%%%%%%%%%%%%%%% READ MIL DATASET %%%%%%%%%%%%%%%%%%%%%%
nameBenchmark = 'musk1'; %'musk1','musk2'

bagInstances = textread([ 'MIL_Data/' nameBenchmark '.data'],'','headerlines',3);
bagLabels = textread([ 'MIL_Data/' nameBenchmark '-bag.labels'],'%d','headerlines',3)
instancesBagIdx = textread([ 'MIL_Data/' nameBenchmark '-instance.index'],'%d','headerlines',3)+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% DIVIDE TRAIN/TEST BAGS %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nBags = numel(bagLabels);
trainBagsIdx = sort(randperm(nBags,ceil(nBags/2)));
testBagsIdx = setdiff(1:nBags,trainBagsIdx);

trainInstanceIdx = find(ismember(instancesBagIdx,trainBagsIdx));
testInstanceIdx = find(ismember(instancesBagIdx,testBagsIdx));

trainBagLabels = bagLabels(trainBagsIdx);
testBagLabels = bagLabels(testBagsIdx);
trainBagInstances = bagInstances(trainInstanceIdx,:);
testBagInstances = bagInstances(testInstanceIdx,:);
trainInstancesBagIdx = instancesBagIdx(trainInstanceIdx);
testInstancesBagIdx = instancesBagIdx(testInstanceIdx);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% Train/Test MIL, MC-MIL and RMC-MIL %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%  MIL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
params.r = 6; %sqrt of the generalized mean function parameter(1=artihmetic mean / larger=approximates max function)
params.beta = Inf; %No Regularization!
params.k = 1; % One concept
model1 = RMCMIL_Train( trainBagInstances, trainBagLabels,ot_continuousIdx(trainInstancesBagIdx), params );
[predictedBagLabel1] = RMCMIL_Test(model1,testBagInstances,ot_continuousIdx(testInstancesBagIdx)');

%%%%%%%%%%%%%%%%%%%%%%%% MC-MIL %%%%%%%%%%%%%%%%%%%%%%%%%%%%
params.beta = Inf; % No Regularization!
params.k = 10; % %Multi-Concept!
model2 = RMCMIL_Train( trainBagInstances, trainBagLabels,ot_continuousIdx(trainInstancesBagIdx), params );
[predictedBagLabel2] = RMCMIL_Test(model2,testBagInstances,ot_continuousIdx(testInstancesBagIdx)');

%%%%%%%%%%%%%%%%%%%%%%%  RMC-MIL  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
params.beta = 0.25; %Sparse-Regularization!
params.k = 10; % Multi-Concept
model3 = RMCMIL_Train( trainBagInstances, trainBagLabels,ot_continuousIdx(trainInstancesBagIdx), params );
[predictedBagLabel3] = RMCMIL_Test(model3,testBagInstances,ot_continuousIdx(testInstancesBagIdx)');
           

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% Show experiment results %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
display(' ')
display('----------------------  MIL  -----------------------------------')
display(['Accuracy: ' num2str(mean(predictedBagLabel1==testBagLabels'))])
display(['Concepts common sparsity: ' num2str(mean(sum(model1.Z,2).^2<10e-5))])
display('---------------------- MC-MIL ----------------------------------')
display(['Accuracy: ' num2str(mean(predictedBagLabel2==testBagLabels'))])
display(['Concepts common sparsity: ' num2str(mean(sum(model2.Z,2).^2<10e-5))])
display('---------------------- RMC-MIL ---------------------------------')
display(['Accuracy: ' num2str(mean(predictedBagLabel3==testBagLabels'))])
display(['Concepts common sparsity: ' num2str(mean(sum(model3.Z,2).^2<10e-5))])
display('----------------------------------------------------------------')

